#ifndef HEXDECASC
#define HEXDECASC
#include "Arduino.h"

char hexDigit();
char rtnASCIIcode();
char *formatByteAsHex();
char *formatWordAsHex();
char *formatWordAsDec();
char *formatRamDump();




/* --- This returns the character for a Hex Digit --- */
char hexDigit(byte n)                          // this can be used for oct, decimal, bcd or hex digits       
  { if (n>15)                                  // we will except no wine before its time 
      { return (63);}                          // or any digit above 15: '?' 
    else if (n>9)                              // if greater than nine need an ALPHA charater
      { return (n + 0x37);}                    // Hex(10) + Hex(37)=Hex(41), Hex(41) = Decimal(65) = 'A'  
     else                                      // else we need a digit
      { return (n + 0x30);}                    // Hex(00) + Hex(30)=Hex(30), Hex(30) = Decimal(48) = '0'
  }

/* --- Filter characters codes. This allows only defined ASCII characters ------------------ */
char rtnASCIIcode(byte n)         
  { if      (n==92) {return '.';}              // avoid chance of escape sequence by blocking "\"
    else if (n< 32) {return '.';}              // non-printing control characters
    else if (n>126) {return '.';}              // 8 bit characters are undefined by ASCII
    else { return char(n);}                    // that leaves the all the rest
  }


// --- Return the HEX representation of a Byte --- (demo returning char array from function)

/* Of special note in this first function is that we are returning a char array from a function. I had 
   some difficulting in trying to find an example of this for the Arduino. There are three key points:
   
     1) You should pass the array to the function. If you try to declare the array inside the function
        it will be lost when the function exists. Another option is to use the optional "Static" key word
        when the char[] is declared inside the function. This alocates space much like declaring a global 
        variable. The advantage to static is that it is not allocated until the function is called the 
        first time. The static variable will also retain its value between calls to the function.
        
     2) You are really returning a pointer to the array, not the array itself. A pointer is something like
        a road sign that tells you where to go to find the array. Thus the declaration must include "*"
        dereference pointer operator. This is one of the pointer operators that the Arduino reference guide
        indicates you may never need, but this case you need it. To use it you simply append the "*" to the
        front of your funtion name. 
        
     3) Return the pointer by using the keyword return and the name of the char array.
*/ 
// -----------
char *formatByteAsHex(byte n, char tmp[])      // this function formats 'n' as two hex digits
  { tmp[1] = hexDigit(n & B00001111);          // in the char array 'tmp' that is passed to it.
    tmp[0] = hexDigit(n >> 4);                 // it returns that the pointer to that char array
    return tmp;                                // temp MUST BE at least two bytes long
  }
char *formatByteAsHex(char tmp[] )             // this function 'overloads' the previous one
  { return(formatByteAsHex(byte(tmp[0]), tmp));}

  
// ----------  
char *formatWordAsHex(word w, char tmp[])      // this function formats 'n' as four hex digits
  { formatByteAsHex(lowByte(w),& tmp[2]);      // temp MUST BE at least four bytes long
    return formatByteAsHex(highByte(w),tmp);   
  }
char *formatWordAsHex(char tmp[] )             // this function 'overloads' the previous one
  { return(formatWordAsHex(word(tmp[0],tmp[1]), tmp));}

// ------------
char *formatWordAsDec(word n, char tmp[])      // this function formats 'n' as five Dec digits
  { tmp[4] = hexDigit(n %10);                  // Use intger math to get each digit: 54321 % 10 = 1
    tmp[3] = hexDigit((n % 100)/10);           // 54321 % 100 = 21, 21/10=2 
    tmp[2] = hexDigit((n % 1000)/100);         // 54321 % 1000 = 321, 321/100=3
    tmp[1] = hexDigit((n % 10000)/1000);       // 54321 % 10000= 4321, 4321/1000=4
    tmp[0] = hexDigit(n/10000);                // 54321/10000=5
    return tmp;                                // temp MUST BE at least five bytes long
  }
char *formatWordAsDec(char tmp[] )             // this function 'overloads' the previous one
  { return(formatWordAsDec(word(tmp[0],tmp[1]), tmp));}

//------------
char *formatRamDump(char hd, word addr, char data[], char buffer[])   
  // this function formats the address and data into the buffer with the ASCII representation of:
  //       address in Hex (hd='h' or 'H') or default as Decimal
  //       data in HEX   (MUST BE 16 bytes)
  //       data in ASCII
  //  buffer array must be at least 60 bytes
  //  0         1         2         3         4         5         6 
  //  01234567890123456789012345678901234567890123456789012345678901
  //  00000  HLHLHLHL HLHLHLHL HLHLHLHL HLHLHLHL  AAAAAAAAAAAAAAAA\n
  { byte i,p,s;
    buffer[60]=0;                                // stuff a null terminator 
    for (i=0; i<16; i++)                         // do ASCII first
      { buffer[i+44]=rtnASCIIcode(data[i]);}     // get the ASCII code
    buffer[42]=' ';                              // two spaces
    buffer[43]=' ';
    i=15;                                        // do HEX next and do it backwards
    s=0;                                         // counter for seperator
    for (p=40; p>6  ; p=p-2)                     // four bytes at a time
      { formatByteAsHex(data[i--],& buffer[p]);  // get Hig/low hex characters 
        if (++s == 4)                            // if we have done 4 bytes  
         { buffer[--p]=' ';                      // add a space
           s=0;                                  // and reset the counter
         }                                       // do it again sam
      }
    buffer[5]=' ';                               // two spaces
    buffer[6]=' ';
    if (hd == 'h' || hd == 'H')                  // if H or h was sent then use HEX for address
      { formatWordAsHex(addr, buffer);
        buffer[4]='H';                           // Flag as HEX format
      }
    else                                         // otherwise use decimal 
      { formatWordAsDec(addr, buffer);}
    return buffer;                               // done, return the char array pointer   
  }
char *formatRamDump(word addr, char data[], char buffer[]) // overload for default decimal
  { return formatRamDump('D', addr, data, buffer);}
char *formatRamDump(word addr,char buffer[])               // overload decimal, data=buffer
  { return formatRamDump('D', addr, & buffer[0], buffer);}

#endif 

